#~ '(C) Links System Software 2009
#~ 'Feedback information
#~ 'www: http://lss2008.livejournal.com/
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru
#~ 'icq: 328-958-369

#~ sublend.rb ver. 1.3 14-Aug-09
#~ Little plug-in, which creates interpolated lines between two existing ones

#~ How to use:
#~ 1)	put sublend.rb to Google SketchUp 6\Plugins folder
#~ 2)	start SketchUp 6
#~ 3)	draw 2 curves then select them
#~ 4)	choose Plugins>> Blend command from the main menu
#~ 5)	Blend Settings input box appears, specify "Steps count" (how many intermediate lines you want to generate)
#~      and click Ok

class Lss_SU_Blend
    def initialize (steps=10)
      @steps=steps
    end
    def blend(steps,draw_surf_flag)
      mesh_group = Sketchup.active_model.entities.add_group
      contours_group = Sketchup.active_model.entities.add_group
      @steps=steps
      @model=Sketchup.active_model
      @selection=@model.selection
      @selected_type=@selection[0].typename
      entities = Sketchup.active_model.entities
      points1=[]
      points2=[]
      case  @selected_type
        when "Edge"
          edges1=@selection[0].all_connected
          edges2=@selection[0].all_connected
          ind=0
          while edges2==edges1
            edges2=@selection[ind].all_connected
            ind +=1
          end
          if not edges1[0].curve.nil?
            points1=edges1[0].curve.vertices 
            points2=edges2[0].curve.vertices
          else
            points1=[edges1[0].start,edges1[0].end]
            points2=[edges2[0].start,edges2[0].end]
          end
        when "Face"
          face1=@selection[0]
          face2=@selection[1]
          points1=face1.vertices
          points2=face2.vertices
        end
     
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      step=0
      while step<=steps+1
        pnt_ind=0
        verts=[]
        while pnt_ind<max_pnts_cnt
          pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
          pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
          vert1=points1[pnt_ind1].position
          vert2=points2[pnt_ind2].position
          new_vert=[0,0,0]
          new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps+1)
          new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps+1)
          new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps+1)
          #~ verts.each{|vert|
            verts += [new_vert] #if not vert==new_vert
          #~ }
          pnt_ind +=1
        end
        new_curve=contours_group.entities.add_curve verts if @selected_type=="Edge" and not draw_surf_flag=="Surface only (suppress contours)"
        new_face=contours_group.entities.add_face verts if @selected_type=="Face" and not draw_surf_flag=="Surface only (suppress contours)"
        step +=1
        if (draw_surf_flag=="Yes" or draw_surf_flag=="Surface only (suppress contours)") and step<=steps+1
          pnt_ind=0
          verts1=[]
          while pnt_ind<max_pnts_cnt
            pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
            pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
            vert1=points1[pnt_ind1].position
            vert2=points2[pnt_ind2].position
            new_vert=[0,0,0]
            new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps+1)
            new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps+1)
            new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps+1)
            #~ verts1.each{|vert|
              verts1 += [new_vert] #if not vert==new_vert
            #~ }
            pnt_ind +=1
          end
          draw_surf verts,verts1,mesh_group,@selected_type
        end
      end
      mesh=Geom::PolygonMesh.new
      mesh_group.entities.each{|face|
        pt1=face.vertices[0].position if face.vertices[0]
        pt2=face.vertices[1].position if face.vertices[1]
        pt3=face.vertices[2].position if face.vertices[2]
        mesh.add_polygon pt1,pt2,pt3 if (pt1 and pt2 and pt3)
      }
      mesh_group.entities.clear!
      mesh_group.entities.add_faces_from_mesh mesh, 12
    end
    
    def draw_surf (points1, points2,group, face_flag)
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      pnt_ind=-1 if face_flag=="Face"
      pnt_ind=0 if not face_flag=="Face"
      while pnt_ind<max_pnts_cnt
        pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).truncate
        pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).truncate
        vert1=points1[pnt_ind1]
        vert2=points2[pnt_ind2]
        pnt_ind3=((pnt_ind+1)*points1.length/max_pnts_cnt).truncate
        pnt_ind4=((pnt_ind+1)*points2.length/max_pnts_cnt).truncate
        vert3=points1[pnt_ind3]
        vert4=points2[pnt_ind4]
        group.entities.add_face vert1,vert2,vert3 if (not vert1.nil? and not vert2.nil? and not vert3.nil? and not vert1==vert3)
        group.entities.add_face vert3,vert4,vert2 if (not vert2.nil? and not vert4.nil? and not vert3.nil? and not vert2==vert4)
        pnt_ind +=1
      end
    end
  end
  
tool_menu = UI.menu "Plugins" 
 tool_menu.add_item("Blend") {
  selection=Sketchup.active_model.selection
  if selection.count<2
    UI.messagebox "It is necessary to select two curves or two faces or two lines."
    return
  end
  if selection[0].typename=="Edge"
    edges=selection[0].all_connected
    if edges.length==selection.count
      UI.messagebox "It is necessary to select two curves"
      return
    end
  end
  model = Sketchup.active_model 
  status = model.start_operation "Blend"
  
  prompts = ["Steps count: ", "Connect steps with surface "]
  defaults= [10, "No"]
  list = ["", "No|Yes|Surface only (suppress contours)"]
  settings=UI.inputbox prompts, defaults, list, "Blend Settings"
  lss_blend=Lss_SU_Blend.new
  lss_blend.blend settings[0], settings[1]
  
  status = model.commit_operation
}